package com.qmth.themis.task.quartz.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.qmth.themis.business.cache.ExamRecordCacheUtil;
import com.qmth.themis.business.cache.RedisKeyHelper;
import com.qmth.themis.business.cache.bean.ExamCacheBean;
import com.qmth.themis.business.cache.bean.ExamStudentCacheBean;
import com.qmth.themis.business.entity.TEExamActivity;
import com.qmth.themis.business.enums.ExamRecordStatusEnum;
import com.qmth.themis.business.enums.FinishTypeEnum;
import com.qmth.themis.business.service.TEExamActivityService;
import com.qmth.themis.business.service.TEExamService;
import com.qmth.themis.business.service.TEExamStudentService;
import com.qmth.themis.business.util.RedisUtil;
import com.qmth.themis.task.quartz.service.QuartzLogicService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.Map;
import java.util.Objects;

/**
 * @Description: quartz逻辑 impl
 * @Param:
 * @return:
 * @Author: wangliang
 * @Date: 2020/7/31
 */
@Service
public class QuartzLogicServiceImpl implements QuartzLogicService {
    private final static Logger log = LoggerFactory.getLogger(QuartzLogicServiceImpl.class);

    @Resource
    TEExamActivityService teExamActivityService;

    @Resource
    TEExamService teExamService;

    @Resource
    RedisUtil redisUtil;

    @Resource
    TEExamStudentService teExamStudentService;

    /**
     * 考试场次quartz逻辑
     *
     * @param key
     */
    @Override
    @Transactional
    public void execExamActivityJobLogic(String key) {
        log.info("key:{}", key);
        String[] strings = key.split("\\.");
        QueryWrapper<TEExamActivity> teExamActivityQueryWrapper = new QueryWrapper<>();
        teExamActivityQueryWrapper.lambda().eq(TEExamActivity::getCode, strings[1]);
        TEExamActivity teExamActivity = teExamActivityService.getOne(teExamActivityQueryWrapper);
        ExamCacheBean examCacheBean = teExamService.getExamCacheBean(teExamActivity.getExamId());
        if (Objects.nonNull(examCacheBean) && examCacheBean.getEnable() == 1 && Objects.nonNull(teExamActivity) && teExamActivity.getEnable() == 1) {
            Map<String, Object> objectMap = redisUtil.getHashEntries(RedisKeyHelper.examActivityRecordCacheKey(teExamActivity.getId()));
            if (Objects.nonNull(objectMap) && objectMap.size() > 0) {
                objectMap.forEach((k, v) -> {
                    ExamRecordStatusEnum examRecordStatusEnum = (ExamRecordStatusEnum) v;
                    //获取该考试批次下所有未交卷的考生的考试记录
                    if (Objects.nonNull(examRecordStatusEnum) && !Objects.equals(ExamRecordStatusEnum.FINISHED, examRecordStatusEnum)) {
                        Long recordId = Long.parseLong(k);
                        Integer durationSeconds = ExamRecordCacheUtil.getDurationSeconds(recordId);
                        Long examStudentId = ExamRecordCacheUtil.getExamStudentId(recordId);
                        ExamStudentCacheBean examStudentCacheBean = teExamStudentService.getExamStudentCacheBean(examStudentId);
                        //交卷
                        teExamService.finish(examStudentCacheBean.getStudentId(), Long.parseLong(k), FinishTypeEnum.AUTO.name(), durationSeconds);
                    }
                });
            }
        }
    }
}
